@@ -30,15 +30,17 @@
 #include "nm-client.h"
 #include "nm-device.h"
 #include "nm-device-modem.h"
-#include <libnm-gtk/nm-ui-utils.h>
+#include "nm-ui-utils.h"
 
 #include "net-device.h"
 #include "net-device-mobile.h"
 #include "net-device-wifi.h"
 #include "net-device-ethernet.h"
 #include "net-device-bond.h"
+#include "net-device-bridge.h"
 #include "net-object.h"
 #include "net-proxy.h"
+#include "net-virtual-device.h"
 #include "net-vpn.h"
 
 #include "rfkill-glib.h"
@@ -46,6 +48,7 @@
 #include "panel-common.h"
 
 #include "network-dialogs.h"
+#include "connection-editor/net-connection-editor.h"
 
 CC_PANEL_REGISTER (CcNetworkPanel, cc_network_panel)
 
@@ -98,7 +101,7 @@ enum {
         PROP_ARGV
 };
 
-static NetObject *find_in_model_by_id (CcNetworkPanel *panel, const gchar *id);
+static NetObject *find_in_model_by_id (CcNetworkPanel *panel, const gchar *id, GtkTreeIter *iter_out);
 static void handle_argv (CcNetworkPanel *panel);
 
 static void
@@ -599,6 +602,28 @@ handle_argv (CcNetworkPanel *panel)
         g_debug ("Could not handle argv operation, no matching device yet?");
 }
 
+static void
+state_changed_cb (NMDevice *device,
+                  NMDeviceState new_state,
+                  NMDeviceState old_state,
+                  NMDeviceStateReason reason,
+                  CcNetworkPanel *panel)
+{
+        GtkListStore *store;
+        GtkTreeIter iter;
+
+        if (!find_in_model_by_id (panel, nm_device_get_udi (device), &iter)) {
+                return;
+        }
+
+        store = GTK_LIST_STORE (gtk_builder_get_object (panel->priv->builder,
+                                                        "liststore_devices"));
+
+        gtk_list_store_set (store, &iter,
+                            PANEL_DEVICES_COLUMN_ICON, panel_device_to_icon_name (device),
+                           -1);
+}
+
 static gboolean
 panel_add_device (CcNetworkPanel *panel, NMDevice *device)
 {
@@ -612,7 +637,7 @@ panel_add_device (CcNetworkPanel *panel, NMDevice *device)
         GType device_g_type;
 
         /* do we have an existing object with this id? */
-        if (find_in_model_by_id (panel, nm_device_get_udi (device)) != NULL)
+        if (find_in_model_by_id (panel, nm_device_get_udi (device), NULL) != NULL)
                 goto out;
 
         type = nm_device_get_device_type (device);
@@ -632,8 +657,9 @@ panel_add_device (CcNetworkPanel *panel, NMDevice *device)
                 device_g_type = NET_TYPE_DEVICE_WIFI;
                 break;
         case NM_DEVICE_TYPE_BOND:
-                device_g_type = NET_TYPE_DEVICE_BOND;
-                break;
+        case NM_DEVICE_TYPE_BRIDGE:
+        case NM_DEVICE_TYPE_VLAN:
+                goto out;
         default:
                 device_g_type = NET_TYPE_DEVICE_SIMPLE;
                 break;
@@ -672,6 +698,8 @@ panel_add_device (CcNetworkPanel *panel, NMDevice *device)
                             PANEL_DEVICES_COLUMN_SORT, panel_device_to_sortable_string (device),
                             PANEL_DEVICES_COLUMN_OBJECT, net_device,
                             -1);
+        g_signal_connect (device, "state-changed",
+                          G_CALLBACK (state_changed_cb), panel);
 
 out:
         return FALSE;
@@ -736,8 +764,13 @@ panel_add_devices_columns (CcNetworkPanel *panel, GtkTreeView *treeview)
 
         /* image */
         renderer = gtk_cell_renderer_pixbuf_new ();
-        g_object_set (renderer, "stock-size", gtk_icon_size_from_name ("cc-sidebar-list"), NULL);
-        gtk_cell_renderer_set_padding (renderer, 4, 4);
+        g_object_set (renderer,
+                      "width", 32,
+                      "xalign", 1.0,
+                      "stock-size", GTK_ICON_SIZE_MENU,
+                      "follow-state", TRUE,
+                      NULL);
+        gtk_cell_renderer_set_padding (renderer, 4, 10);
 
         column = gtk_tree_view_column_new_with_attributes ("icon", renderer,
                                                            "icon-name", PANEL_DEVICES_COLUMN_ICON,
@@ -838,7 +871,7 @@ panel_add_proxy_device (CcNetworkPanel *panel)
         gtk_list_store_append (liststore_devices, &iter);
         gtk_list_store_set (liststore_devices,
                             &iter,
-                            PANEL_DEVICES_COLUMN_ICON, "preferences-system-network",
+                            PANEL_DEVICES_COLUMN_ICON, "preferences-system-network-symbolic",
                             PANEL_DEVICES_COLUMN_SORT, "9",
                             PANEL_DEVICES_COLUMN_OBJECT, proxy,
                             -1);
@@ -939,7 +972,7 @@ out:
 }
 
 static NetObject *
-find_in_model_by_id (CcNetworkPanel *panel, const gchar *id)
+find_in_model_by_id (CcNetworkPanel *panel, const gchar *id, GtkTreeIter *iter_out)
 {
         gboolean ret;
         NetObject *object_tmp;
@@ -968,6 +1001,8 @@ find_in_model_by_id (CcNetworkPanel *panel, const gchar *id)
                 }
         } while (object == NULL && gtk_tree_model_iter_next (model, &iter));
 out:
+        if (iter_out)
+                *iter_out = iter;
         return object;
 }
 
@@ -984,7 +1019,7 @@ panel_add_vpn_device (CcNetworkPanel *panel, NMConnection *connection)
 
         /* does already exist */
         id = nm_connection_get_path (connection);
-        if (find_in_model_by_id (panel, id) != NULL)
+        if (find_in_model_by_id (panel, id, NULL) != NULL)
                 return;
 
         /* add as a virtual object */
@@ -994,6 +1029,7 @@ panel_add_vpn_device (CcNetworkPanel *panel, NMConnection *connection)
                                 "id", id,
                                 "connection", connection,
                                 "client", panel->priv->client,
+                                "remote-settings", panel->priv->remote_settings,
                                 NULL);
         g_signal_connect_object (net_vpn, "removed",
                                  G_CALLBACK (object_removed_cb), panel, 0);
@@ -1015,29 +1051,98 @@ panel_add_vpn_device (CcNetworkPanel *panel, NMConnection *connection)
         gtk_list_store_append (liststore_devices, &iter);
         gtk_list_store_set (liststore_devices,
                             &iter,
-                            PANEL_DEVICES_COLUMN_ICON, "network-vpn",
+                            PANEL_DEVICES_COLUMN_ICON, "network-vpn-symbolic",
                             PANEL_DEVICES_COLUMN_SORT, "5",
                             PANEL_DEVICES_COLUMN_OBJECT, net_vpn,
                             -1);
         g_free (title);
 }
 
+static void
+panel_add_virtual_device (CcNetworkPanel *panel, NMConnection *connection)
+{
+        gchar *title;
+        GtkListStore *liststore_devices;
+        GtkTreeIter iter;
+        NetVirtualDevice *net_virt;
+        const gchar *id;
+        GtkNotebook *notebook;
+        GtkSizeGroup *size_group;
+        NMSettingConnection *s_con;
+        const gchar *connection_type;
+        GType device_g_type;
+
+        /* does already exist */
+        id = nm_connection_get_path (connection);
+        if (find_in_model_by_id (panel, id, NULL) != NULL)
+                return;
+
+        /* map the NMConnection to a NetDevice GType */
+        s_con = nm_connection_get_setting_connection (connection);
+        connection_type = nm_setting_connection_get_connection_type (s_con);
+        if (!strcmp (connection_type, NM_SETTING_BOND_SETTING_NAME))
+                device_g_type = NET_TYPE_DEVICE_BOND;
+        else if (!strcmp (connection_type, NM_SETTING_BRIDGE_SETTING_NAME))
+                device_g_type = NET_TYPE_DEVICE_BRIDGE;
+        else
+                device_g_type = NET_TYPE_VIRTUAL_DEVICE;
+
+        /* add as a virtual object */
+        net_virt = g_object_new (device_g_type,
+                                 "panel", panel,
+                                 "removable", TRUE,
+                                 "id", id,
+                                 "connection", connection,
+                                 "client", panel->priv->client,
+                                 NULL);
+        g_signal_connect_object (net_virt, "removed",
+                                 G_CALLBACK (object_removed_cb), panel, 0);
+
+        /* add as a panel */
+        notebook = GTK_NOTEBOOK (gtk_builder_get_object (panel->priv->builder,
+                                                         "notebook_types"));
+        size_group = GTK_SIZE_GROUP (gtk_builder_get_object (panel->priv->builder,
+                                                             "sizegroup1"));
+        net_object_add_to_notebook (NET_OBJECT (net_virt),
+                                    notebook,
+                                    size_group);
+
+        liststore_devices = GTK_LIST_STORE (gtk_builder_get_object (panel->priv->builder,
+                                            "liststore_devices"));
+        title = nma_utils_get_connection_device_name (connection);
+
+        net_object_set_title (NET_OBJECT (net_virt), title);
+        gtk_list_store_append (liststore_devices, &iter);
+        gtk_list_store_set (liststore_devices,
+                            &iter,
+                            PANEL_DEVICES_COLUMN_ICON, "network-wired",
+                            PANEL_DEVICES_COLUMN_SORT, "2",
+                            PANEL_DEVICES_COLUMN_OBJECT, net_virt,
+                            -1);
+        g_free (title);
+}
+
 static void
 add_connection (CcNetworkPanel *panel,
                 NMConnection *connection)
 {
         NMSettingConnection *s_con;
-        const gchar *type;
+        const gchar *type, *iface;
 
         s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection,
                                                                   NM_TYPE_SETTING_CONNECTION));
         type = nm_setting_connection_get_connection_type (s_con);
-        if (g_strcmp0 (type, "vpn") != 0)
+        iface = nm_connection_get_virtual_iface_name (connection);
+        if (g_strcmp0 (type, "vpn") != 0 && iface == NULL)
                 return;
+
         g_debug ("add %s/%s remote connection: %s",
                  type, g_type_name_from_instance ((GTypeInstance*)connection),
                  nm_connection_get_path (connection));
-        panel_add_vpn_device (panel, connection);
+        if (iface)
+                panel_add_virtual_device (panel, connection);
+        else
+                panel_add_vpn_device (panel, connection);
 }
 
 static void
@@ -1123,46 +1228,26 @@ panel_check_network_manager_version (CcNetworkPanel *panel)
         return ret;
 }
 
+static void
+editor_done (NetConnectionEditor *editor,
+             gboolean             success,
+             gpointer             user_data)
+{
+        g_object_unref (editor);
+}
+
 static void
 add_connection_cb (GtkToolButton *button, CcNetworkPanel *panel)
 {
-        GtkWidget *dialog;
-        gint response;
+        NetConnectionEditor *editor;
+        GtkWindow *toplevel;
 
-        dialog = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder,
-                                                     "connection_type_dialog"));
-        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (panel))));
-
-        response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-        gtk_widget_hide (dialog);
-
-        if (response == GTK_RESPONSE_OK) {
-                GtkComboBox *combo;
-                GtkTreeModel *model;
-                GtkTreeIter iter;
-                gchar *type;
-                gchar *cmdline;
-                GError *error;
-
-                combo = GTK_COMBO_BOX (gtk_builder_get_object (panel->priv->builder,
-                                                               "connection_type_combo"));
-                model = gtk_combo_box_get_model (combo);
-                gtk_combo_box_get_active_iter (combo, &iter);
-                type = NULL;
-                gtk_tree_model_get (model, &iter, 1, &type, -1);
-
-                cmdline = g_strdup_printf ("nm-connection-editor --create --type %s", type);
-                g_debug ("Launching '%s'\n", cmdline);
-
-                error = NULL;
-                if (!g_spawn_command_line_async (cmdline, &error)) {
-                        g_warning ("Failed to launch nm-connection-editor: %s", error->message);
-                        g_error_free (error);
-                }
-                g_free (cmdline);
-                g_free (type);
-        }
+        toplevel = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (panel)));
+        editor = net_connection_editor_new (toplevel, NULL, NULL, NULL,
+                                            panel->priv->client,
+                                            panel->priv->remote_settings);
+        g_signal_connect (editor, "done", G_CALLBACK (editor_done), panel);
+        net_connection_editor_run (editor);
 }
 
 static void
