@@ -119,13 +119,19 @@ nm_device_simple_refresh_ui (NetDeviceSimple *device_simple)
                                 && state != NM_DEVICE_STATE_UNMANAGED);
         update_off_switch_from_device_state (GTK_SWITCH (widget), state, device_simple);
 
+        /* set up the Options button */
+        widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_options"));
+        gtk_widget_set_visible (widget, state != NM_DEVICE_STATE_UNMANAGED);
+
         /* set device state, with status and optionally speed */
         widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_status"));
         status = g_string_new (panel_device_state_to_localized_string (nm_device));
         if (state != NM_DEVICE_STATE_UNAVAILABLE)
                 speed = net_device_simple_get_speed (device_simple);
         if (speed) {
-                g_string_append_printf (status, " - %s", speed);
+                if (status->len)
+                        g_string_append (status, " - ");
+                g_string_append (status, speed);
                 g_free (speed);
         }
         gtk_label_set_label (GTK_LABEL (widget), status->str);
@@ -207,7 +213,7 @@ net_device_simple_constructed (GObject *object)
 
         G_OBJECT_CLASS (net_device_simple_parent_class)->constructed (object);
 
-        nm_device_simple_refresh_ui (device_simple);
+        net_object_refresh (NET_OBJECT (device_simple));
 }
 
 static void
