@@ -44,26 +44,26 @@ panel_device_to_icon_name (NMDevice *device)
         switch (nm_device_get_device_type (device)) {
         case NM_DEVICE_TYPE_ETHERNET:
                 state = nm_device_get_state (device);
-                if (state == NM_DEVICE_STATE_UNAVAILABLE) {
-                        value = "network-wired-disconnected";
+                if (state <= NM_DEVICE_STATE_DISCONNECTED) {
+                        value = "network-wired-disconnected-symbolic";
                 } else {
-                        value = "network-wired";
+                        value = "network-wired-symbolic";
                 }
                 break;
         case NM_DEVICE_TYPE_WIFI:
         case NM_DEVICE_TYPE_BT:
         case NM_DEVICE_TYPE_OLPC_MESH:
-                value = "network-wireless";
+                value = "network-wireless-signal-excellent-symbolic";
                 break;
         case NM_DEVICE_TYPE_MODEM:
                 caps = nm_device_modem_get_current_capabilities (NM_DEVICE_MODEM (device));
                 if ((caps & NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS) ||
                     (caps & NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO)) {
-                        value = "network-wireless";
+                        value = "network-wireless-signal-excellent-symbolic";
                 }
                 break;
         default:
-                value = "network-idle";
+                value = "network-idle-symbolic";
                 break;
         }
         return value;
@@ -156,18 +156,20 @@ panel_device_state_to_localized_string (NMDevice *device)
                 value = _("Unmanaged");
                 break;
         case NM_DEVICE_STATE_UNAVAILABLE:
-                /* TRANSLATORS: device status */
-                if (nm_device_get_firmware_missing (device))
+                if (nm_device_get_firmware_missing (device)) {
+                        /* TRANSLATORS: device status */
                         value = _("Firmware missing");
-                else if (type == NM_DEVICE_TYPE_ETHERNET &&
-                         !nm_device_ethernet_get_carrier (NM_DEVICE_ETHERNET (device)))
+                } else if (type == NM_DEVICE_TYPE_ETHERNET &&
+                           !nm_device_ethernet_get_carrier (NM_DEVICE_ETHERNET (device))) {
+                        /* TRANSLATORS: device status */
                         value = _("Cable unplugged");
-                else
+                } else {
+                        /* TRANSLATORS: device status */
                         value = _("Unavailable");
+                }
                 break;
         case NM_DEVICE_STATE_DISCONNECTED:
-                /* TRANSLATORS: device status */
-                value = _("Disconnected");
+                value = "";
                 break;
         case NM_DEVICE_STATE_PREPARE:
         case NM_DEVICE_STATE_CONFIG:
@@ -488,8 +490,8 @@ panel_set_device_widget_header (GtkBuilder *builder,
         return TRUE;
 }
 
-static gchar *
-get_ipv4_config_address_as_string (NMIP4Config *ip4_config, const char *what)
+gchar *
+panel_get_ip4_address_as_string (NMIP4Config *ip4_config, const char *what)
 {
         const GSList *list;
         struct in_addr addr;
@@ -522,8 +524,8 @@ out:
         return str;
 }
 
-static gchar *
-get_ipv4_config_name_servers_as_string (NMIP4Config *ip4_config)
+gchar *
+panel_get_ip4_dns_as_string (NMIP4Config *ip4_config)
 {
         const GArray *array;
         GString *dns;
@@ -547,8 +549,8 @@ out:
         return str;
 }
 
-static gchar *
-get_ipv6_config_address_as_string (NMIP6Config *ip6_config)
+gchar *
+panel_get_ip6_address_as_string (NMIP6Config *ip6_config)
 {
         const GSList *list;
         const struct in6_addr *addr;
@@ -586,7 +588,7 @@ panel_set_device_widgets (GtkBuilder *builder, NMDevice *device)
         if (ip4_config != NULL) {
 
                 /* IPv4 address */
-                str_tmp = get_ipv4_config_address_as_string (ip4_config, "address");
+                str_tmp = panel_get_ip4_address_as_string (ip4_config, "address");
                 panel_set_device_widget_details (builder,
                                                  "ipv4",
                                                  str_tmp);
@@ -594,14 +596,14 @@ panel_set_device_widgets (GtkBuilder *builder, NMDevice *device)
                 g_free (str_tmp);
 
                 /* IPv4 DNS */
-                str_tmp = get_ipv4_config_name_servers_as_string (ip4_config);
+                str_tmp = panel_get_ip4_dns_as_string (ip4_config);
                 panel_set_device_widget_details (builder,
                                                  "dns",
                                                  str_tmp);
                 g_free (str_tmp);
 
                 /* IPv4 route */
-                str_tmp = get_ipv4_config_address_as_string (ip4_config, "gateway");
+                str_tmp = panel_get_ip4_address_as_string (ip4_config, "gateway");
                 panel_set_device_widget_details (builder,
                                                  "route",
                                                  str_tmp);
@@ -628,7 +630,7 @@ panel_set_device_widgets (GtkBuilder *builder, NMDevice *device)
         /* get IPv6 parameters */
         ip6_config = nm_device_get_ip6_config (device);
         if (ip6_config != NULL) {
-                str_tmp = get_ipv6_config_address_as_string (ip6_config);
+                str_tmp = panel_get_ip6_address_as_string (ip6_config);
                 panel_set_device_widget_details (builder, "ipv6", str_tmp);
                 has_ip6 = str_tmp != NULL;
                 g_free (str_tmp);
