@@ -879,7 +879,7 @@ static const SECTION union { const guint8 data[6960]; const double alignment; vo
   0x2f, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00
 } };
 
-static GStaticResource static_resource = { cc_online_accounts_resource_data.data, sizeof (cc_online_accounts_resource_data.data) };
+static GStaticResource static_resource = { cc_online_accounts_resource_data.data, sizeof (cc_online_accounts_resource_data.data), NULL, NULL, NULL };
 extern GResource *cc_online_accounts_get_resource (void);
 GResource *cc_online_accounts_get_resource (void)
 {
